<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2006 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

/**
 * アクションの基底クラスです。
 * 全てのアクションクラスはこのクラスを継承して実装してください。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Action
{
    /**
     * Moony_Requestのインスタンス
     * @var object
     */
    var $request;

    /**
     * Moony_Sessionのインスタンス
     * @var object
     */
    var $session;

    /**
     * Moony_Cacheのインスタンス
     * @var object
     */
    var $cache;

    /**
     * Moony_Smartyのインスタンス
     * @var object
     */
    var $smarty;

    /**
     * アクションの処理を実行します。
     * 実装時にオーバーライドしてください。
     * オーバーライドしない場合、テンプレートの処理結果を出力します。
     *
     * @access public
     */
    function execute()
    {
        $this->render();
    }

    /**
     * テンプレートの処理結果を取得します。
     * $templateが設定されていない場合、
     * 使用するテンプレートをアクションパスを元に決定します。
     * テンプレートファイルが存在しない場合、falseを返します。
     * PHPテンプレートを使用する場合、テンプレートの変数スコープは
     * このメソッドと同一です。$local_varsが設定されている場合、
     * 値は展開されてローカル変数として扱うことかできます。
     * Smartyテンプレートを使用する場合、
     * メンバ変数は「action」という名前でアサインされます。
     * すなわち、メンバ変数$this->fooはテンプレート内部で{$action.foo}と
     * アクセスできるようになります。$local_varsが設定されている場合、
     * 値は展開されてそのままアサインされます。
     *
     * @access public
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @return string|bool テンプレートの処理結果
     */
    function fetch($template = null, $local_vars = null)
    {
        $moony =& Moony_Registry::get('_moony');
        if (is_null($template)) {
            $action_path = $this->request->getActionPath();
            $ext = $moony->use_smarty ? '.tpl' : '.php';
            $template = strtolower($action_path) . $ext;
        }
        if (is_null($moony->template_dir)) {
            // error
            if (ob_get_length() !== false) {
                ob_end_clean();
            }
            trigger_error('Not defined: template_dir', E_USER_ERROR);
            exit;
        }

        $result = null;
        if ($moony->use_smarty) {
            // with Smarty
            if (!$this->smarty->template_exists($template)) {
                return false;
            }
            $this->smarty->assign('action', (array) $this);
            if (is_array($local_vars)) {
                $this->smarty->assign($local_vars);
            }
            $result = $this->smarty->fetch($template);
        } else {
            // without Smarty
            $path = $moony->template_dir . DIRECTORY_SEPARATOR . $template;
            if (!file_exists($path)) {
                return false;
            }
            if (is_array($local_vars)) {
                extract($local_vars, EXTR_SKIP);
            }
            ob_start();
            ob_implicit_flush(false);
            include $path;
            $result = ob_get_contents();
            ob_end_clean();
        }
        return $result;
    }

    /**
     * テンプレートの処理結果を出力します。
     * $templateが設定されていない場合、
     * 使用するテンプレートをアクションパスを元に決定します。
     * テンプレートファイルが存在しない場合、
     * HTTPのステータスで404を送出して処理を終了します。
     * PHPテンプレートを使用する場合、テンプレートの変数スコープは
     * このメソッドと同一です。$local_varsが設定されている場合、
     * 値は展開されてローカル変数として扱うことかできます。
     * Smartyテンプレートを使用する場合、
     * メンバ変数は「action」という名前でアサインされます。
     * すなわち、メンバ変数$this->fooはテンプレート内部で{$action.foo}と
     * アクセスできるようになります。$local_varsが設定されている場合、
     * 値は展開されてそのままアサインされます。
     * $convert_encodingがtrueで出力エンコーディングが指定されている場合、
     * 出力エンコーディングに変換された内容が出力されます。
     *
     * @access public
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function render(
        $template = null, $local_vars = null, $convert_encoding = true)
    {
        $result = $this->fetch($template, $local_vars);
        if ($result === false) {
            // not found
            ob_end_clean();
            $this->sendStatus(404);
            exit;
        }
        $this->renderText($result, $convert_encoding);
    }

    /**
     * テキストを出力します。
     * $convert_encodingがtrueで出力エンコーディングが指定されている場合、
     * 出力エンコーディングに変換された内容が出力されます。
     *
     * @access public
     * @param string $text 出力するテキスト
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function renderText($text, $convert_encoding = true)
    {
        $moony =& Moony_Registry::get('_moony');
        if ($convert_encoding) {
            if (!is_null($moony->output_encoding)) {
                $text = mb_convert_encoding($text,
                    $moony->output_encoding, $moony->internal_encoding);
            }
        }
        echo $text;
    }

    /**
     * 値をJSON形式に変換して出力します。
     * このメソッドを正常に使用するためには、
     * JSONエクステンションまたはJsphonが利用可能である必要があります。
     * いずれもインストールされていない場合、警告を発生、falseを返します。
     * $mime_typeはHTTPヘッダのContent-Typeとして送出されます。
     *
     * @access public
     * @param mixed $value JSONに変換して出力する値
     * @param string $mime_type 出力時に使用されるMIMEタイプ
     * @see http://pecl.php.net/package/json
     * @see http://www.hawklab.jp/jsonencoder/
     * @see http://www.rfc-editor.org/rfc/rfc4627.txt
     */
    function renderJson($value, $mime_type = 'application/json')
    {
        // to UTF-8
        $moony =& Moony_Registry::get('_moony');
        if ($moony->internal_encoding != 'UTF-8') {
            mb_convert_variables('UTF-8', $moony->internal_encoding, $value);
        }

        $json = null;
        if (function_exists('json_encode')) {
            // use JSON extension
            $json = json_encode($value);
        } else if (class_exists('Jsphon')) {
            // use Jsphon
            $json = Jsphon::encode($value);
        } else {
            trigger_error(
                'Not loaded: JSON extension or Jsphon', E_USER_WARNING);
            return false;
        }

        if (!is_null($mime_type)) {
            header("Content-Type: {$mime_type}");
        }
        header('Content-Length: ' . strlen($json));
        echo $json;
    }

    /**
     * 任意の画像を出力します。
     * 指定されたファイルが存在しないか、
     * ファイルを読み込むことができない場合にfalseを返します。
     * （使用例: $this->renderImage('/path/to/file', 'image/jpeg');）
     *
     * @access public
     * @param string $path 描画するファイル
     * @param string $content_type MIMEタイプ（image/jpegなど）
     * @param int $buffer_size ファイル読み込みサイズ
     * @return null|bool ファイルが存在しない場合false
     */
    function renderImage($path, $content_type, $buffer_size = 4096)
    {
        $path = realpath($path);
        if (!file_exists($path) || !is_file($path)) {
            return false;
        }
        $fp = fopen($path, 'rb');
        if ($fp === false) {
            return false;
        }

        header("Content-Type: {$content_type}");
        header('Content-Disposition: inline');
        header('Content-Length: ' . filesize($path));

        while (!feof($fp)) {
            echo fread($fp, $buffer_size);
        }
        fclose($fp);
    }

    /**
     * リダイレクト処理を行い、処理を終了します。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param string $url リダイレクト先のURL
     * @param int $status ステータスコード
     */
    function redirect($url, $status = null)
    {
        if (ob_get_length() !== false) {
            ob_end_clean();
        }
        if (!is_null($status)) {
            $this->sendStatus($status);
        }
        header("Location: {$url}");
        exit;
    }

    /**
     * 指定された名称のアクションにリダイレクト処理を行い、
     * 処理を終了します。
     * $action_paramsが設定されていた場合、
     * URLの末尾にPATH_INFOとして付加、パラメータとして渡します。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param string $action_name リダイレクト先のアクション名
     * @param array $action_params アクションに渡すパラメータの配列
     * @param int $status ステータスコード
     */
    function redirectAction(
        $action_name, $action_params = null, $status = null)
    {
        $action_path = strtolower(strtr($action_name, '_', '/'));
        $url = $this->request->getHostUrl()
             . $this->request->getActionBase()
             . '/' . $action_path;
        if (is_array($action_params)) {
            foreach ($action_params as $param) {
                $url .= '/' . urlencode($param);
            }
        }
        $this->redirect($url, $status);
    }

    /**
     * 検証エラーが存在する場合、
     * テンプレートの処理結果を出力して処理を終了します。
     * $templateが設定されていない場合、
     * 使用するテンプレートをアクションパスを元に決定します。
     * テンプレートファイルが存在しない場合、
     * HTTPのステータスで404を送出して処理を終了します。
     * PHPテンプレートを使用する場合、テンプレートの変数スコープは
     * このメソッドと同一です。$local_varsが設定されている場合、
     * 値は展開されてローカル変数として扱うことかできます。
     * Smartyテンプレートを使用する場合、
     * メンバ変数は「action」という名前でアサインされます。
     * すなわち、メンバ変数$this->fooはテンプレート内部で{$action.foo}と
     * アクセスできるようになります。$local_varsが設定されている場合、
     * 値は展開されてそのままアサインされます。
     * $convert_encodingがtrueで出力エンコーディングが指定されている場合、
     * 出力エンコーディングに変換された内容が出力されます。
     * 検証エラーが存在しない場合、何も処理されません。
     *
     * @access public
     * @param bool|object $v 検証結果、またはMoony_Validatorのインスタンス
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function renderOnError($v,
        $template = null, $local_vars = null, $convert_encoding = true)
    {
        if ($v === false || $v->hasError()) {
            $this->render($template, $local_vars, $convert_encoding);
            exit;
        }
    }

    /**
     * 検証エラーが存在する場合、
     * リダイレクト処理を行って処理を終了します。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param bool|object $v 検証結果、またはMoony_Validatorのインスタンス
     * @param string $url リダイレクト先のURL
     * @param int $status ステータスコード
     */
    function redirectOnError($v, $url, $status = null)
    {
        if ($v === false || $v->hasError()) {
            $this->redirect($url, $status);
            exit;
        }
    }

    /**
     * 検証エラーが存在する場合、
     * 指定された名称のアクションにリダイレクト処理を行い、
     * 処理を終了します。
     * $action_paramsが設定されていた場合、
     * URLの末尾にPATH_INFOとして付加、パラメータとして渡します。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param bool|object $v 検証結果、またはMoony_Validatorのインスタンス
     * @param string $action_name リダイレクト先のアクション名
     * @param array $action_params アクションに渡すパラメータの配列
     * @param int $status ステータスコード
     */
    function redirectActionOnError(
        $v, $action_name, $action_params = null, $status = null)
    {
        if ($v === false || $v->hasError()) {
            $this->redirectAction($action_name, $action_params, $status);
            exit;
        }
    }

    /**
     * HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param int $status ステータスコード
     * @see http://www.rfc-editor.org/rfc/rfc1945.txt
     * @see http://www.rfc-editor.org/rfc/rfc2616.txt
     */
    function sendStatus($status)
    {
        // HTTP/1.1?
        $protocol = $this->request->getServer('SERVER_PROTOCOL');
        $is_http_11 = (strpos($protocol, '1.1') !== false);
        // messages
        $messages = array(
            // Informational 1xx
            100 => 'Continue',
            101 => 'Switching Protocols',
            // Success 2xx
            200 => 'OK',
            201 => 'Created',
            202 => 'Accepted',
            203 => 'Non-Authoritative Information',
            204 => 'No Content',
            205 => 'Reset Content',
            206 => 'Partial Content',
            // Redirection 3xx
            300 => 'Multiple Choices',
            301 => 'Moved Permanently',
            302 => 'Found',  // 1.1
            303 => 'See Other',
            304 => 'Not Modified',
            305 => 'Use Proxy',
            // 306 is no longer used but still reserved
            307 => 'Temporary Redirect',
            // Client Error 4xx
            400 => 'Bad Request',
            401 => 'Unauthorized',
            402 => 'Payment Required',
            403 => 'Forbidden',
            404 => 'Not Found',
            405 => 'Method Not Allowed',
            406 => 'Not Acceptable',
            407 => 'Proxy Authentication Required',
            408 => 'Request Timeout',
            409 => 'Conflict',
            410 => 'Gone',
            411 => 'Length Required',
            412 => 'Precondition Failed',
            413 => 'Request Entity Too Large',
            414 => 'Request-URI Too Long',
            415 => 'Unsupported Media Type',
            416 => 'Requested Range Not Satisfiable',
            417 => 'Expectation Failed',
            // Server Error 5xx
            500 => 'Internal Server Error',
            501 => 'Not Implemented',
            502 => 'Bad Gateway',
            503 => 'Service Unavailable',
            504 => 'Gateway Timeout',
            505 => 'HTTP Version Not Supported',
            509 => 'Bandwidth Limit Exceeded'
        );
        if (isset($messages[$status])) {
            if (!$is_http_11) {
                $messages[302] = 'Moved Temporarily';
            }
            $http_version = $is_http_11 ? '1.1' : '1.0';
            header("HTTP/{$http_version} {$status} {$messages[$status]}");
        }
    }

    /**
     * 任意のファイルをダウンロードさせます。
     * $nameが設定されていない場合、元のファイル名を使用します。
     * 指定されたファイルが存在しないか、
     * ファイルを読み込むことができない場合にfalseを返します。
     * 使用例: $this->sendFile('/path/to/foo.zip');
     *
     * @access public
     * @param string $path ダウンロードするファイルのパス
     * @param string $name ダウンロードするファイルにつける名前
     * @param int $buffer_size ファイル読み込み時のバッファサイズ
     * @return null|bool ファイルが存在しない場合false
     */
    function sendFile($path, $name = null, $buffer_size = 4096)
    {
        $path = realpath($path);
        if (!file_exists($path) || !is_file($path)) {
            return false;
        }
        if (is_null($name)) {
            // 名前が指定されていない場合、元のファイル名を使用
            $name = basename($path);
        }
        $fp = fopen($path, 'rb');
        if ($fp === false) {
            return false;
        }

        header('Content-Type: application/octet-stream');
        header("Content-Disposition: attachment; filename=\"{$name}\"");
        header('Content-Length: ' . filesize($path));

        while (!feof($fp)) {
            echo fread($fp, $buffer_size);
        }
        fclose($fp);
    }

    /**
     * アップロードされたファイルの保存を行います。
     * 保存先のディレクトリは前もって用意されている必要があります。
     * なお、配列を用いた複数ファイルのアップロードには対応していません。
     *
     * @access public
     * @param string $name アップロードファイル名
     * @param string $path 保存先のパス
     * @param int $mode 保存先のファイルのモード
     * @return bool アップロードに成功したかどうか
     */
    function moveFile($name, $path, $mode = 0666)
    {
        $file_info = $this->request->getFile($name);
        if (is_null($file_info)) {
            return false;
        }
        $dir = dirname($path);
        if (!file_exists($dir) || !is_dir($dir)) {
            // ディレクトリが存在しない
            return false;
        }
        if ($file_info['error'] != UPLOAD_ERR_OK) {
            // アップロード時エラー
            return false;
        }
        if (move_uploaded_file($file_info['tmp_name'], $path)) {
            // アップロードファイルの移動、モード変更
            chmod($path, $mode);
            return true;
        }
        return false;
    }

    /**
     * Moonyの動作設定を変更します。
     * テンプレートファイルを格納するディレクトリを設定します。
     *
     * @access public
     * @param string $template_dir テンプレートファイルを格納するディレクトリ
     */
    function setTemplateDir($template_dir)
    {
        $moony =& Moony_Registry::get('_moony');
        $moony->setTemplateDir($template_dir);
        if (!is_null($this->smarty)) {
            $this->smarty->template_dir = $moony->template_dir;
        }
    }

    /**
     * Moonyの動作設定を変更します。
     * 出力エンコーディングを設定します。
     * 設定した場合、出力時のデフォルトエンコーディングとして使用されます。
     *
     * @access public
     * @param string $output_encoding 出力エンコーディング
     */
    function setOutputEncoding($output_encoding)
    {
        $moony =& Moony_Registry::get('_moony');
        $moony->setOutputEncoding($output_encoding);
    }

    /**
     * Moonyの動作設定を変更します。
     * 変更はprepareメソッド内で実行されない限り有効になりません。
     * 出力バッファリングのハンドラを設定します。
     * 設定した場合、アクションクラスのconvert、validate、executeの
     * 各メソッドが実行される間の
     * 出力バッファリングのハンドラとして用いられます。
     *
     * @access public
     * @param callbacl $ob_handler 出力バッファリングのハンドラ
     */
    function setObHandler($ob_handler)
    {
        $moony =& Moony_Registry::get('_moony');
        $moony->setObHandler($ob_handler);
    }

    /**
     * Moonyの動作設定を変更します。
     * 変更はprepareメソッド内で実行されない限り有効になりません。
     * セッションのオプション設定を設定します。
     * オプションには以下の指定が可能です:
     *   - name: セッション名
     *   - id: セッションID
     *   - save_path: セッションデータの保存先
     *   - cache_expire: カレントのキャッシュの有効期限
     *   - cache_limiter: カレントのキャッシュリミッタ
     *
     * @access public
     * @param array $options セッションのオプション設定
     */
    function setSessionOptions($options)
    {
        $moony =& Moony_Registry::get('_moony');
        $moony->setSessionOptions($options);
    }

    /**
     * Moonyの動作設定を変更します。
     * 変更はprepareメソッド内で実行されない限り有効になりません。
     * セッションを自動的に開始するかどうかを設定します。
     * 初期設定値は「true」で、変更しない限り
     * 自動的にセッションが開始されます。
     *
     * @access public
     * @param bool $session_auto_start セッションを自動で開始するかどうか
     */
    function setSessionAutoStart($session_auto_start)
    {
        $moony =& Moony_Registry::get('_moony');
        $moony->setSessionAutoStart($session_auto_start);
    }
}

/**
 * 別のテンプレートファイルを読み込んで表示するヘルパー関数です。
 * 該当のテンプレートファイルが存在しない場合、何も処理しません。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php moony_include('foo.php'); ?>）
 *
 * @access public
 * @param string $template テンプレートファイル
 */
function moony_include($template)
{
    static $action;
    if (is_null($action)) {
        $action = Moony_Registry::get('_moony_action');
    }
    $content = $action->fetch($template);
    if ($content !== false) {
        echo $content;
    }
}

/**
 * 全ての検証エラーメッセージを表示するヘルパー関数です。
 * エラーメッセージが設定されていない場合、何も処理しません。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php moony_errors(); ?>）
 *
 * @access public
 * @param string $header 全体のヘッダ
 * @param string $footer 全体のフッタ
 * @param string $prefix 個々のメッセージの接頭辞
 * @param string $postfix 個々のメッセージの接尾辞
 */
function moony_errors($header = '<ul>',
    $footer = '</ul>', $prefix = '<li>', $postfix = '</li>')
{
    $validator =& Moony_Registry::get('_moony_validator');
    if (!is_null($validator)) {
        $error_messages = $validator->getErrorMessages();
        if (is_array($error_messages) && count($error_messages) > 0) {
            echo $header;
            foreach ($error_messages as $error_message) {
                echo $prefix, $error_message, $postfix;
            }
            echo $footer;
        }
    }
}

/**
 * 検証エラーの有無を判断するヘルパー関数です。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php if (moony_has_error()): ?>）
 *
 * @access public
 * @return bool 検証エラーの存在有無
 */
function moony_has_error()
{
    $validator =& Moony_Registry::get('_moony_validator');
    if (!is_null($validator)) {
        return $validator->hasError();
    }
    return false;
}

/**
 * 文字列をエスケープした上で出力するヘルパー関数です。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php h($this->foo); ?>）
 *
 * @access public
 * @param string $string 出力したい文字列
 */
function h($string)
{
    static $internal_encoding;
    if (is_null($internal_encoding)) {
        $moony =& Moony_Registry::get('_moony');
        $internal_encoding = $moony->internal_encoding;
    }
    echo htmlspecialchars($string, ENT_QUOTES, $internal_encoding);
}
?>
