<?php
/**
 * Moony - the tiny web application framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2006 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

/**
 * セッションを扱うクラスです。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Session
{
    /**
     * セッションが開始されているかどうか
     * @var bool
     */
    var $_is_started = false;

    /**
     * セッションの値を取得します。
     * 該当する値が存在しない場合、$defaultを返します。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     * @param string $name 値の名称
     * @param mixed $default 値が存在しない場合のデフォルト値
     * @return mixed セッションの値
     */
    function get($name, $default = null)
    {
        if (!$this->_is_started) {
            return false;
        }
        if (array_key_exists($name, $_SESSION)) {
            return $_SESSION[$name];
        }
        return $default;
    }

    /**
     * セッションに値を設定します。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     * @param string $name 値の名称
     * @param mixed $value セッションに設定する値
     */
    function set($name, $value)
    {
        if (!$this->_is_started) {
            return false;
        }
        $_SESSION[$name] = $value;
    }

    /**
     * セッションに値が保管されているかどうかを調べます。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     * @param string $name 値の名称
     * @return bool セッションに値が保管されているかどうか
     */
    function exists($name)
    {
        if (!$this->_is_started) {
            return false;
        }
        return array_key_exists($name, $_SESSION);
    }

    /**
     * セッションから指定された名称の値を除去します。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     * @param string $name 除去する値の名称
     * @return mixed 除去された値
     */
    function remove($name)
    {
        if (!$this->_is_started) {
            return false;
        }
        if (array_key_exists($name, $_SESSION)) {
            $value = $_SESSION[$name];
            unset($_SESSION[$name]);
            return $value;
        }
    }

    /**
     * セッションから全ての値を除去します。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     */
    function clear()
    {
        if (!$this->_is_started) {
            return false;
        }
        $_SESSION = array();
    }

    /**
     * 一時的な保管領域を扱います。
     * $valueが設定されていない場合、値を取得します。
     * 取得した値はセッションから除去されます。
     * $valueが設定されている場合、値を保管します。
     * 保管された値は次に取得されるまで有効です。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     * @param string $name 値の名称
     * @param mixed $value 保管する値
     * @return mixed 保管されていた値
     */
    function flash($name, $value = null)
    {
        if (!$this->_is_started) {
            return false;
        }
        if (is_null($value)) {
            if (isset($_SESSION['_moony_flash'][$name])) {
                $value = $_SESSION['_moony_flash'][$name];
                unset($_SESSION['_moony_flash'][$name]);
                return $value;
            }
        } else {
            $_SESSION['_moony_flash'][$name] = $value;
        }
    }

    /**
     * セッションを開始します。
     * 既に開始されている場合、falseを返します。
     *
     * @access public
     */
    function start()
    {
        if ($this->_is_started) {
            return false;
        }
        session_start();
        $this->_is_started = true;
    }

    /**
     * セッションを終了します。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     */
    function close()
    {
        if (!$this->_is_started) {
            return false;
        }
        session_write_close();
        $this->_is_started = false;
    }

    /**
     * セッションを破棄します。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     */
    function destroy()
    {
        if (!$this->_is_started) {
            return false;
        }
        $_SESSION = array();
        session_destroy();
        $this->_is_started = false;
    }

    /**
     * セッション名を設定します。
     * 既にセッションが開始されている場合、falseを返します。
     *
     * @access public
     * @param string $name セッション名
     */
    function setName($name)
    {
        if ($this->_is_started) {
            return false;
        }
        session_name($name);
    }

    /**
     * セッションIDを設定します。
     * 既にセッションが開始されている場合、falseを返します。
     *
     * @access public
     * @param string $id セッションID
     */
    function setId($id)
    {
        if ($this->_is_started) {
            return false;
        }
        session_id($id);
    }

    /**
     * セッションデータの保存先を設定します。
     * 既にセッションが開始されている場合、falseを返します。
     *
     * @access public
     * @param string $save_path セッションデータの保存先
     */
    function setSavePath($save_path)
    {
        if ($this->_is_started) {
            return false;
        }
        session_save_path($save_path);
    }

    /**
     * ユーザ定義のセッションハンドラを設定します。
     * 既にセッションが開始されている場合、falseを返します。
     *
     * @access public
     * @param callback $open セッションを開始する際に実行される関数
     * @param callback $close セッションを終了する際に実行される関数
     * @param callback $read セッションの値を取得する際に実行される関数
     * @param callback $write セッションに値を書き込む際に実行される関数
     * @param callback $destroy セッションを破棄する際に実行される関数
     * @param callback $gc 不要になったセッションデータを削除する関数
     * @return bool セッションハンドラの設定に成功したか
     */
    function setSaveHandler($open, $close, $read, $write, $destroy, $gc)
    {
        if ($this->_is_started) {
            return false;
        }
        return session_set_save_handler(
            $open, $close, $read, $write, $destroy, $gc);
    }

    /**
     * カレントのキャッシュの有効期限（単位:分）を設定します。
     * 既にセッションが開始されている場合、falseを返します。
     *
     * @access public
     * @param int $cache_expire カレントのキャッシュの有効期限
     */
    function setCacheExpire($cache_expire)
    {
        if ($this->_is_started) {
            return false;
        }
        session_cache_expire($cache_expire);
    }

    /**
     * カレントのキャッシュリミッタを設定します。
     * 既にセッションが開始されている場合、falseを返します。
     *
     * @access public
     * @param string $cache_limiter カレントのキャッシュリミッタ
     */
    function setCacheLimiter($cache_limiter)
    {
        if ($this->_is_started) {
            return false;
        }
        session_cache_limiter($cache_limiter);
    }

    /**
     * セッションクッキーの有効期限を設定します。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     * @param int $lifetime セッションクッキーの有効期限
     */
    function setCookieLifetime($lifetime)
    {
        if (!$this->_is_started) {
            return false;
        }
        $params = session_get_cookie_params();
        session_set_cookie_params($lifetime,
            $params['path'], $params['domain'], $params['secure']);
    }

    /**
     * セッションクッキーで設定するパスを設定します。
     *
     * @access public
     * @param string $path セッションクッキーで設定するパス
     */
    function setCookiePath($path)
    {
        if (!$this->_is_started) {
            return false;
        }
        $params = session_get_cookie_params();
        session_set_cookie_params($params['lifetime'],
            $path, $params['domain'], $params['secure']);
    }

    /**
     * セッションクッキーで指定するドメインを設定します。
     *
     * @access public
     * @param string $domain セッションクッキーで指定するドメイン
     */
    function setCookieDomain($domain)
    {
        if (!$this->_is_started) {
            return false;
        }
        $params = session_get_cookie_params();
        session_set_cookie_params($params['lifetime'],
            $params['path'], $domain, $params['secure']);
    }

    /**
     * セッションクッキーをSSL経由でのみ送信できるかどうかを設定します。
     *
     * @access public
     * @param bool $is_secure セッションクッキーをSSL経由でのみ送信できるか
     */
    function setCookieSecure($is_secure)
    {
        if (!$this->_is_started) {
            return false;
        }
        $params = session_get_cookie_params();
        session_set_cookie_params($params['lifetime'],
            $params['path'], $params['domain'], (bool) $is_secure);
    }

    /**
     * セッションが開始されているかどうかを調べます。
     *
     * @access public
     * @return bool セッションが開始されているかどうか
     */
    function isStarted()
    {
        return $this->_is_started;
    }

    /**
     * セッションIDを再生成します。
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     */
    function regenerateId()
    {
        if (!$this->_is_started) {
            return false;
        }
        if (version_compare(PHP_VERSION, '5.1.0') >= 0) {
            session_regenerate_id(true);
            return;
        }
        $tmp = $_SESSION;
        $this->destory();
        session_id(md5(uniqid(rand(), true)));
        $this->start();
        $_SESSION = $tmp;
    }
}
?>
